package gui;

/**
 * 
 * Copyright 2007 - Dirk Becker, Germany
 * 
 * This Code ist FreeWare and OpenSource, feel free to Copy or Modify it!
 * 
 *  Commercial use is absolutely forbidden!
 *  This entail the use by someone who act for free in order of an commercial oriented company.
 *  Please stick to this rules!
 * 
 */
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumnModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeModel;
import main.TPLE_Editor;
import main.data.Converter;
import main.data.TPLE_Class;
import main.data.TPLE_Value;
import main.enums.AIMode;
import main.enums.Action;
import main.enums.AmbientAction;
import main.enums.AniState;
import main.enums.AttackReason;
import main.enums.Attitude;
import main.enums.AttribModOperation;
import main.enums.AttribReqOperation;
import main.enums.Bearing;
import main.enums.CastType;
import main.enums.CollisionGroup;
import main.enums.DamageType;
import main.enums.DirectionalShadowType;
import main.enums.DynamicLightEffect;
import main.enums.EnclaveStatus;
import main.enums.EventType;
import main.enums.FinalBlend;
import main.enums.Focus;
import main.enums.FocusLookAt;
import main.enums.FocusLookAtKeysFOR;
import main.enums.FocusNameType;
import main.enums.FocusPriority;
import main.enums.Gender;
import main.enums.GuardStatus;
import main.enums.HitDirection;
import main.enums.ItemCategory;
import main.enums.LastFightAgainstPlayer;
import main.enums.LightmapAmbientOcclusion;
import main.enums.LightmapType;
import main.enums.NPCType;
import main.enums.PartyMemberType;
import main.enums.PhysicRangeType;
import main.enums.PlayerCrime;
import main.enums.PoliticalAlignment;
import main.enums.PropertySetType;
import main.enums.ShapeGroup;
import main.enums.ShapeMaterial;
import main.enums.SkillCategory;
import main.enums.Slot;
import main.enums.Species;
import main.enums.SpellDeity;
import main.enums.SpellTarget;
import main.enums.SpellType;
import main.enums.StaticIlluminated;
import main.enums.StaticLighingType;
import main.enums.StripSpawning;
import main.enums.TradeCategory;
import main.enums.TreasureDistribution;
import main.enums.UseType;

public class ContentPane extends JSplitPane implements ListSelectionListener {

	private static final long serialVersionUID = 3500504187828091120L;
	
	private TreeView tree;
	private JTable itemTable;
	private JPanel hexEditPanel;
	private TPLE_Editor MainApp;
	private boolean mechChange = false;

	public ContentPane(TPLE_Editor app) {
		super(JSplitPane.HORIZONTAL_SPLIT);
		MainApp = app;
		setDividerLocation(200);
		setLeftComponent(getTree());
		setRightComponent(getEditorPanel());
	}

	private JComponent getTree() {
		tree = new TreeView();
		tree.setBorder(BorderFactory.createEmptyBorder());
		tree.setScrollsOnExpand(true);
		tree.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {}
			public void mouseEntered(MouseEvent e) {}
			public void mouseExited(MouseEvent e) {}
			public void mousePressed(MouseEvent e) {}
			public void mouseReleased(MouseEvent e) {
				if(tree.getLastSelectedPathComponent()!=null) {
					if(tree.getSelectionRows()[0]==0) {
						MainApp.setSelectedClass(null);
						setItemHash();
						MainApp.getStatusBar().setStatusText("<html><body><strong>Item-Hash:</strong> " +
								Converter.byteToHexString(MainApp.getCurrentFile().getItemHash()) +
								"<br /><strong>Haendler-Hash:</strong> " +
								Converter.byteToHexString(MainApp.getCurrentFile().getHaendlerHash()) + "</body></html>");
					}
					else {
						MainApp.setSelectedClass((TPLE_Class)((DefaultMutableTreeNode)tree.getLastSelectedPathComponent()).getUserObject());
						MainApp.getStatusBar().setStatusText("<html><body><strong>G3 Template Editor</strong> - Not for commercial use !!</body></html>");
					}
					fillTableData();
				}
			}
		});

		JScrollPane scrlPane = new JScrollPane(tree);
		scrlPane.setBorder(BorderFactory.createEmptyBorder());
		return scrlPane;
	}

	private HexField[] hexFields = new HexField[32];
	
	private JSplitPane rightSplit;
	private JScrollPane scrlHex;
	private JScrollPane scrlTable;

	private JComponent getEditorPanel() {
		rightSplit = new JSplitPane(VERTICAL_SPLIT);
		rightSplit.setDividerSize(0);
		
		hexEditPanel = new JPanel(new GridLayout(2,0));

		// Hex-Edit Felder erzeugen und einfgen
		for(int it=0; it<hexFields.length; it++) {
			hexFields[it] = new HexField();
			hexFields[it].setEnabled(false);
			hexFields[it].setHorizontalAlignment(TextField.CENTER);
			hexFields[it].setPreferredSize(new Dimension(25, 25));
			hexFields[it].setMaximumSize(new Dimension(25, 25));
			hexFields[it].setMinimumSize(new Dimension(25, 25));
			hexFields[it].getDocument().addDocumentListener(new HexFieldListener(MainApp, it, hexFields[it]));
			hexEditPanel.add(hexFields[it]);
		}
		scrlHex = new JScrollPane(hexEditPanel);
		scrlHex.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrlHex.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
		scrlHex.setPreferredSize(new Dimension(scrlHex.getPreferredSize().width, 52));
		scrlHex.setMaximumSize(new Dimension(scrlHex.getPreferredSize().width, 52));
		scrlHex.setMinimumSize(new Dimension(scrlHex.getPreferredSize().width, 52));
		scrlHex.setVisible(false);
		rightSplit.setLeftComponent(scrlHex);

		itemTable = new JTable();
		itemTable.setOpaque(true);
		itemTable.setBackground(Color.WHITE);
		itemTable.getSelectionModel().addListSelectionListener(this);
		scrlTable = new JScrollPane(itemTable);
		scrlTable.setBorder(BorderFactory.createEmptyBorder());
		rightSplit.setRightComponent(scrlTable);

		return rightSplit;
	}
	
	private void reorderEditorPanel() {
		rightSplit.setDividerLocation(scrlHex.isVisible() ? 52 : 0);
		rightSplit.repaint();
	}
	
	public void setItemHash() {
		setMechChange(true);
		// Alle Felder ersteinmal wieder ausblenden
		for(HexField tf: hexFields) { tf.setEnabled(false); tf.setText(""); }

		String[] hexString = Converter.byteToHex(MainApp.getCurrentFile().getItemHash());
		for(int it=0; it<hexString.length; it++) {
			hexFields[it].setEnabled(true);
			hexFields[it].setText(hexString[it]);
		}

		String[] hexString2 = Converter.byteToHex(MainApp.getCurrentFile().getHaendlerHash());
		for(int it=16; it<hexString2.length+16; it++) {
			hexFields[it].setEnabled(true);
			hexFields[it].setText(hexString2[it-16]);
		}
		hexEditPanel.repaint();
		setMechChange(false);
	}
	
	public void changeHex() {
		setMechChange(true);
		TPLE_Value value = (TPLE_Value)itemTable.getValueAt(itemTable.getSelectedRow(), 0);

		// Alle Felder ersteinmal wieder ausblenden
		for(HexField tf: hexFields) { tf.setEnabled(false); tf.setText(""); }

		String[] hexString = Converter.byteToHex(value.getValue());
		for(int it=0; it<hexString.length; it++) {
			hexFields[it].setEnabled(true);
			hexFields[it].setText(hexString[it]);
		}
		hexEditPanel.repaint();
		setMechChange(false);
	}

	public void valueChanged(ListSelectionEvent e) {
		if(e.getValueIsAdjusting() && itemTable.getRowCount()>0 && itemTable.getSelectedRow()>=0) {
			changeHex();

			TPLE_Value value = (TPLE_Value)itemTable.getValueAt(itemTable.getSelectedRow(), 0);
			MainApp.getStatusBar().setObjectInfo(	Converter.longToHex(value.getPosition()),
													Converter.HTMLEntities(value.getType().getValue()),
													value.getName().getValue(), Converter.byteToHexString(value.getValue()) );
		}
	}

	public JPanel getHexEditPanel() { return hexEditPanel; }
	public JTable getItemTable() { return itemTable; }

	public TreeView getTreeView() { return tree; }
	public void setTreeModel(TreeModel model) { tree.setModel(model); }

	public void fillTableData() { fillTableData(false); }
	public void fillTableData(boolean refresh) {
		int selRow = refresh ? itemTable.getSelectedRow() : -1;

		TPLE_Class selectedClass = MainApp.getSelectedClass();
		
		int i = 0;
		TableColumnModel cmodel = null;
		int[] columnwidth = new int[0];
		String [] header = { "", "Property", "Value" };
		Object [][] entries;

		if(selectedClass!=null) {
			if(!refresh) clearHexFields();

			// save old column-width
			cmodel = itemTable.getTableHeader().getColumnModel();
			columnwidth = new int[cmodel.getColumnCount()];
			for(int column=0; column<cmodel.getColumnCount(); column++) columnwidth[column] = cmodel.getColumn(column).getWidth();

			entries = new Object[selectedClass.getValues().size()][3];
			
			for(TPLE_Value value: selectedClass.getValues()) {
				entries[i][0] = value;
				entries[i][1] = value.getName().getValue();
				switch(value.getDataType()) {
					case BOOLEAN:
						entries[i][2] = value.getBoolean() ? "true" : "false";
						break;
					case SHORT:
						entries[i][2] = value.getShort();
						break;
					case INTEGER:
						entries[i][2] = value.getInteger();
						break;
					case LONG:
						entries[i][2] = value.getLong();
						break;
					case FLOAT:
						entries[i][2] = value.getFloat();
						break;
					case STRING:
					case MESH_RES_STRING:
					case ANIM_RES_STRING:
					case LOC_STRING:
						entries[i][2] = value.getString().getValue();
						break;
					case UNKNOWN_ENUM:
						entries[i][2] = value.getEnumInt();
						break;
					case ITEM_CATEGORY:
						entries[i][2] = ItemCategory.fromValue(value.getValue());
						break;
					case ATTRIB_REQ_OPERATION:
						entries[i][2] = AttribReqOperation.fromValue(value.getValue());
						break;
					case ATTRIB_MOD_OPERATION:
						entries[i][2] = AttribModOperation.fromValue(value.getValue());
						break;
					case FOCUS_PRIORITY:
						entries[i][2] = FocusPriority.fromValue(value.getValue());
						break;
					case USE_TYPE:
						entries[i][2] = UseType.fromValue(value.getValue());
						break;
					case FOCUS_NAME_TYPE:
						entries[i][2] = FocusNameType.fromValue(value.getValue());
						break;
					case ACTION:
						entries[i][2] = Action.fromValue(value.getValue());
						break;
					case AI_MODE:
						entries[i][2] = AIMode.fromValue(value.getValue());
						break;
					case AMBIENT_ACTION:
						entries[i][2] = AmbientAction.fromValue(value.getValue());
						break;
					case ANI_STATE:
						entries[i][2] = AniState.fromValue(value.getValue());
						break;
					case ATTACK_REASON:
						entries[i][2] = AttackReason.fromValue(value.getValue());
						break;
					case ATTITUDE:
						entries[i][2] = Attitude.fromValue(value.getValue());
						break;
					case BEARING:
						entries[i][2] = Bearing.fromValue(value.getValue());
						break;
					case CLASS:
						entries[i][2] = main.enums.Class.fromValue(value.getValue());
						break;
					case DAMAGE_TYPE:
						entries[i][2] = DamageType.fromValue(value.getValue());
						break;
					case FOCUS:
						entries[i][2] = Focus.fromValue(value.getValue());
						break;
					case FOCUS_LOOK_AT:
						entries[i][2] = FocusLookAt.fromValue(value.getValue());
						break;
					case FOCUS_LOOK_AT_KEYS_FOR:
						entries[i][2] = FocusLookAtKeysFOR.fromValue(value.getValue());
						break;
					case GENDER:
						entries[i][2] = Gender.fromValue(value.getValue());
						break;
					case GUARD_STATUS:
						entries[i][2] = GuardStatus.fromValue(value.getValue());
						break;
					case HIT_DIRECTION:
						entries[i][2] = HitDirection.fromValue(value.getValue());
						break;
					case LAST_FIGHT_AGAINST_PLAYER:
						entries[i][2] = LastFightAgainstPlayer.fromValue(value.getValue());
						break;
					case NPC_TYPE:
						entries[i][2] = NPCType.fromValue(value.getValue());
						break;
					case PARTY_MEMBER_TYPE:
						entries[i][2] = PartyMemberType.fromValue(value.getValue());
						break;
					case PLAYER_CRIME:
						entries[i][2] = PlayerCrime.fromValue(value.getValue());
						break;
					case POLITICAL_ALIGNMENT:
						entries[i][2] = PoliticalAlignment.fromValue(value.getValue());
						break;
					case SLOT:
						entries[i][2] = Slot.fromValue(value.getValue());
						break;
					case SPECIES:
						entries[i][2] = Species.fromValue(value.getValue());
						break;
					case TRADE_CATEGORY:
						entries[i][2] = TradeCategory.fromValue(value.getValue());
						break;
					case COLLISION_GROUP:
						entries[i][2] = CollisionGroup.fromValue(value.getValue());
						break;
					case DIRECTIONAL_SHADOW_TYPE:
						entries[i][2] = DirectionalShadowType.fromValue(value.getValue());
						break;
					case DYNAMIC_LIGHT_EFFECT:
						entries[i][2] = DynamicLightEffect.fromValue(value.getValue());
						break;
					case EVENT_TYPE:
						entries[i][2] = EventType.fromValue(value.getValue());
						break;
					case FINAL_BLEND:
						entries[i][2] = FinalBlend.fromValue(value.getValue());
						break;
					case LIGHTMAP_AMBIENT_OCCLUSION:
						entries[i][2] = LightmapAmbientOcclusion.fromValue(value.getValue());
						break;
					case LIGHTMAP_TYPE:
						entries[i][2] = LightmapType.fromValue(value.getValue());
						break;
					case PHYSIC_RANGE_TYPE:
						entries[i][2] = PhysicRangeType.fromValue(value.getValue());
						break;
					case PROPERTY_SET_TYPE:
						entries[i][2] = PropertySetType.fromValue(value.getValue());
						break;
					case SHAPE_GROUP:
						entries[i][2] = ShapeGroup.fromValue(value.getValue());
						break;
					case SHAPE_MATERIAL:
						entries[i][2] = ShapeMaterial.fromValue(value.getValue());
						break;
					case STATIC_ILLUMINATED:
						entries[i][2] = StaticIlluminated.fromValue(value.getValue());
						break;
					case STATIC_LIGHING_TYPE:
						entries[i][2] = StaticLighingType.fromValue(value.getValue());
						break;
					case STRIP_SPAWNING:
						entries[i][2] = StripSpawning.fromValue(value.getValue());
						break;
					case CAST_TYPE:
						entries[i][2] = CastType.fromValue(value.getValue());
						break;
					case ENCLAVE_STATUS:
						entries[i][2] = EnclaveStatus.fromValue(value.getValue());
						break;
					case SKILL_CATEGORY:
						entries[i][2] = SkillCategory.fromValue(value.getValue());
						break;
					case SPELL_DEITY:
						entries[i][2] = SpellDeity.fromValue(value.getValue());
						break;
					case SPELL_TARGET:
						entries[i][2] = SpellTarget.fromValue(value.getValue());
						break;
					case SPELL_TYPE:
						entries[i][2] = SpellType.fromValue(value.getValue());
						break;
					case TREASURE_DISTRIBUTION:
						entries[i][2] = TreasureDistribution.fromValue(value.getValue());
						break;
					default:
						entries[i][2] = "use Hex-Editor";
						break;
				}
				
				i++;
			}
		}
		else entries = new Object[0][8];
		
		setTableModel(header, entries);

		if(cmodel!=null) {
			cmodel.getColumn(0).setMinWidth(0);
			cmodel.getColumn(0).setMaxWidth(0);
			for(int column=1; column<cmodel.getColumnCount(); column++) cmodel.getColumn(column).setPreferredWidth(columnwidth[column]);
		}

		if(selRow>=0) itemTable.setRowSelectionInterval(selRow, selRow);
	}

	private void formatTable() {
		itemTable.getParent().setBackground(Color.WHITE);
		itemTable.setColumnSelectionAllowed(false);
		itemTable.getTableHeader().setReorderingAllowed(false);
		itemTable.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
		itemTable.setShowGrid(true);
		itemTable.setSelectionForeground(Color.BLACK);
		itemTable.setDefaultRenderer(Object.class ,new CustomTableCellRenderer(
				new Integer[] { SwingConstants.WEST, SwingConstants.WEST, SwingConstants.EAST } ));
		itemTable.getTableHeader().setDefaultRenderer(new CustomTableHeadRenderer());
		itemTable.setFont(new java.awt.Font("Verdana", java.awt.Font.PLAIN, 11));
		itemTable.setRowHeight(20);
		itemTable.getTableHeader().getColumnModel().getColumn( 0 ).setMinWidth(0);
		itemTable.getTableHeader().getColumnModel().getColumn( 0 ).setMaxWidth(0);
		itemTable.getColumnModel().getColumn( 0 ).setMinWidth(0);
		itemTable.getColumnModel().getColumn( 0 ).setMaxWidth(0);
		itemTable.getColumnModel().getColumn(2).setCellEditor(new CustomTableCellEditor(MainApp));
	}

	public void setTableModel(String[] header, Object[][] entries) {
		itemTable.setModel(new SortableTableModel(entries, header)  {
			private static final long serialVersionUID = -3437429915700593284L;
			public boolean isCellEditable(int row, int column)  {
				if(column==1) return false;
				else return true;
			}
		});
		formatTable();
	}

	public void clearHexFields() { for(HexField tf: hexFields) { tf.setEnabled(false); tf.setText(""); } }
	public void setHexeditorVisible(boolean boo) {
		scrlHex.setVisible(boo);
		reorderEditorPanel();
	}

	public boolean isMechChange() { return mechChange; }
	public void setMechChange(boolean mechChange) { this.mechChange = mechChange; }
}
