package gui;

/**
 * 
 * Copyright 2007 - Dirk Becker, Germany
 * 
 * This Code ist FreeWare and OpenSource, feel free to Copy or Modify it!
 * 
 *  Commercial use is absolutely forbidden!
 *  This entail the use by someone who act for free in order of an commercial oriented company.
 *  Please stick to this rules!
 * 
 */
import java.awt.Color;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.table.TableCellEditor;

import main.TPLE_Editor;
import main.data.TPLE_Value;
import main.enums.AIMode;
import main.enums.Action;
import main.enums.AmbientAction;
import main.enums.AniState;
import main.enums.AttackReason;
import main.enums.Attitude;
import main.enums.AttribModOperation;
import main.enums.AttribReqOperation;
import main.enums.Bearing;
import main.enums.CastType;
import main.enums.CollisionGroup;
import main.enums.DamageType;
import main.enums.DirectionalShadowType;
import main.enums.DynamicLightEffect;
import main.enums.EnclaveStatus;
import main.enums.EventType;
import main.enums.FinalBlend;
import main.enums.Focus;
import main.enums.FocusLookAt;
import main.enums.FocusLookAtKeysFOR;
import main.enums.FocusNameType;
import main.enums.FocusPriority;
import main.enums.Gender;
import main.enums.GuardStatus;
import main.enums.HitDirection;
import main.enums.ItemCategory;
import main.enums.LastFightAgainstPlayer;
import main.enums.LightmapAmbientOcclusion;
import main.enums.LightmapType;
import main.enums.NPCType;
import main.enums.PartyMemberType;
import main.enums.PhysicRangeType;
import main.enums.PlayerCrime;
import main.enums.PoliticalAlignment;
import main.enums.PropertySetType;
import main.enums.ShapeGroup;
import main.enums.ShapeMaterial;
import main.enums.SkillCategory;
import main.enums.Slot;
import main.enums.Species;
import main.enums.SpellDeity;
import main.enums.SpellTarget;
import main.enums.SpellType;
import main.enums.StaticIlluminated;
import main.enums.StaticLighingType;
import main.enums.StripSpawning;
import main.enums.TradeCategory;
import main.enums.TreasureDistribution;
import main.enums.UseType;

public class CustomTableCellEditor extends AbstractCellEditor implements TableCellEditor {
	private static final long serialVersionUID = 4391227805502104079L;
	
	TPLE_Value selVal;
	private JComboBox comboBox;
	private TextField textBox;
	private IntegerField intBox;
	private TPLE_Editor MainApp;
	
	public CustomTableCellEditor(TPLE_Editor app) { MainApp = app; }

	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
		selVal = (TPLE_Value)table.getValueAt(row, 0);
		
		switch(selVal.getDataType()) {
			case BOOLEAN:
				comboBox = new JComboBox(new Boolean[]{true, false});
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(selVal.getBoolean());
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue((Boolean)comboBox.getSelectedItem());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case SHORT:
				intBox = new IntegerField();
				intBox.setValue((int)selVal.getShort());
				return intBox;
			case INTEGER:
				intBox = new IntegerField();
				intBox.setValue(selVal.getInteger());
				return intBox;
			case LONG:
				intBox = new IntegerField();
				intBox.setValue(selVal.getLong());
				return intBox;
			case FLOAT:
				textBox = new TextField();
				textBox.setBorder(BorderFactory.createEmptyBorder());
				textBox.setHorizontalAlignment(TextField.RIGHT);
				textBox.setText(String.valueOf(selVal.getFloat()));
				textBox.getDocument().addDocumentListener(new DocumentListener() {
					public void changedUpdate(DocumentEvent e) {}
					public void insertUpdate(DocumentEvent e) { changed(); }
					public void removeUpdate(DocumentEvent e) { changed(); }
					private void changed() {
						try {
							float val = Float.parseFloat(textBox.getText());
							selVal.setValue(val);
						}
						catch(NumberFormatException e) {
							Toolkit.getDefaultToolkit().beep();
							selVal.setValue(0F);
						}
						MainApp.getContent().changeHex();
					}
				});
				return textBox;
			case STRING:
			case MESH_RES_STRING:
			case ANIM_RES_STRING:
			case LOC_STRING:
				textBox = new TextField();
				textBox.setBorder(BorderFactory.createEmptyBorder());
				textBox.setHorizontalAlignment(TextField.RIGHT);
				textBox.setText(selVal.getString().getValue());
				return textBox;
			case UNKNOWN_ENUM:
				intBox = new IntegerField();
				intBox.setValue(selVal.getEnumInt());
				return intBox;
			case ITEM_CATEGORY:
				comboBox = new JComboBox(ItemCategory.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(selVal.getBoolean());
				comboBox.setSelectedItem(ItemCategory.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((ItemCategory) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case ATTRIB_REQ_OPERATION:
				comboBox = new JComboBox(AttribReqOperation.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(selVal.getBoolean());
				comboBox.setSelectedItem(AttribReqOperation.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((AttribReqOperation) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case ATTRIB_MOD_OPERATION:
				comboBox = new JComboBox(AttribModOperation.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(selVal.getBoolean());
				comboBox.setSelectedItem(AttribModOperation.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((AttribModOperation) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case FOCUS_PRIORITY:
				comboBox = new JComboBox(FocusPriority.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(selVal.getBoolean());
				comboBox.setSelectedItem(FocusPriority.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((FocusPriority) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case USE_TYPE:
				comboBox = new JComboBox(UseType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(selVal.getBoolean());
				comboBox.setSelectedItem(UseType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((UseType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case FOCUS_NAME_TYPE:
				comboBox = new JComboBox(FocusNameType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(selVal.getBoolean());
				comboBox.setSelectedItem(FocusNameType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((FocusNameType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;

				
			case ACTION:
				comboBox = new JComboBox(Action.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(Action.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((Action) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case AI_MODE:
				comboBox = new JComboBox(AIMode.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(AIMode.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((AIMode) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case AMBIENT_ACTION:
				comboBox = new JComboBox(AmbientAction.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(AmbientAction.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((AmbientAction) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case ANI_STATE:
				comboBox = new JComboBox(AniState.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(AniState.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((AniState) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case ATTACK_REASON:
				comboBox = new JComboBox(AttackReason.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(AttackReason.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((AttackReason) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case ATTITUDE:
				comboBox = new JComboBox(Attitude.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(Attitude.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((Attitude) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case BEARING:
				comboBox = new JComboBox(Bearing.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(Bearing.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((Bearing) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case CLASS:
				comboBox = new JComboBox(main.enums.Class.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(main.enums.Class.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((main.enums.Class) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case DAMAGE_TYPE:
				comboBox = new JComboBox(DamageType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(DamageType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((DamageType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case FOCUS:
				comboBox = new JComboBox(Focus.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(Focus.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((Focus) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case FOCUS_LOOK_AT:
				comboBox = new JComboBox(FocusLookAt.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(FocusLookAt.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((FocusLookAt) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case FOCUS_LOOK_AT_KEYS_FOR:
				comboBox = new JComboBox(FocusLookAtKeysFOR.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(FocusLookAtKeysFOR.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((FocusLookAtKeysFOR) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case GENDER:
				comboBox = new JComboBox(Gender.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(Gender.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((Gender) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case GUARD_STATUS:
				comboBox = new JComboBox(GuardStatus.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(GuardStatus.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((GuardStatus) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case HIT_DIRECTION:
				comboBox = new JComboBox(HitDirection.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(HitDirection.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((HitDirection) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case LAST_FIGHT_AGAINST_PLAYER:
				comboBox = new JComboBox(LastFightAgainstPlayer.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(LastFightAgainstPlayer.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((LastFightAgainstPlayer) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case NPC_TYPE:
				comboBox = new JComboBox(NPCType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(NPCType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((NPCType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case PARTY_MEMBER_TYPE:
				comboBox = new JComboBox(PartyMemberType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(PartyMemberType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((PartyMemberType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case PLAYER_CRIME:
				comboBox = new JComboBox(PlayerCrime.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(PlayerCrime.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((PlayerCrime) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case POLITICAL_ALIGNMENT:
				comboBox = new JComboBox(PoliticalAlignment.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(PoliticalAlignment.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((PoliticalAlignment) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case SLOT:
				comboBox = new JComboBox(Slot.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(Slot.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((Slot) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case SPECIES:
				comboBox = new JComboBox(Species.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(Species.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((Species) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case TRADE_CATEGORY:
				comboBox = new JComboBox(TradeCategory.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(TradeCategory.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((TradeCategory) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case COLLISION_GROUP:
				comboBox = new JComboBox(CollisionGroup.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(CollisionGroup.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((CollisionGroup) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case DIRECTIONAL_SHADOW_TYPE:
				comboBox = new JComboBox(DirectionalShadowType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(DirectionalShadowType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((DirectionalShadowType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case DYNAMIC_LIGHT_EFFECT:
				comboBox = new JComboBox(DynamicLightEffect.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(DynamicLightEffect.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((DynamicLightEffect) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case EVENT_TYPE:
				comboBox = new JComboBox(EventType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(EventType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((EventType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case FINAL_BLEND:
				comboBox = new JComboBox(FinalBlend.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(FinalBlend.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((FinalBlend) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case LIGHTMAP_AMBIENT_OCCLUSION:
				comboBox = new JComboBox(LightmapAmbientOcclusion.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(LightmapAmbientOcclusion.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((LightmapAmbientOcclusion) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case LIGHTMAP_TYPE:
				comboBox = new JComboBox(LightmapType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(LightmapType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((LightmapType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case PHYSIC_RANGE_TYPE:
				comboBox = new JComboBox(PhysicRangeType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(PhysicRangeType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((PhysicRangeType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case PROPERTY_SET_TYPE:
				comboBox = new JComboBox(PropertySetType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(PropertySetType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((PropertySetType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case SHAPE_GROUP:
				comboBox = new JComboBox(ShapeGroup.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(ShapeGroup.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((ShapeGroup) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case SHAPE_MATERIAL:
				comboBox = new JComboBox(ShapeMaterial.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(ShapeMaterial.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((ShapeMaterial) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case STATIC_ILLUMINATED:
				comboBox = new JComboBox(StaticIlluminated.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(StaticIlluminated.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((StaticIlluminated) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case STATIC_LIGHING_TYPE:
				comboBox = new JComboBox(StaticLighingType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(StaticLighingType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((StaticLighingType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case STRIP_SPAWNING:
				comboBox = new JComboBox(StripSpawning.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(StripSpawning.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((StripSpawning) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case CAST_TYPE:
				comboBox = new JComboBox(CastType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(CastType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((CastType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case ENCLAVE_STATUS:
				comboBox = new JComboBox(EnclaveStatus.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(EnclaveStatus.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((EnclaveStatus) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case SKILL_CATEGORY:
				comboBox = new JComboBox(SkillCategory.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(SkillCategory.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((SkillCategory) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case SPELL_DEITY:
				comboBox = new JComboBox(SpellDeity.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(SpellDeity.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((SpellDeity) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case SPELL_TARGET:
				comboBox = new JComboBox(SpellTarget.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(SpellTarget.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((SpellTarget) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case SPELL_TYPE:
				comboBox = new JComboBox(SpellType.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(SpellType.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((SpellType) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			case TREASURE_DISTRIBUTION:
				comboBox = new JComboBox(TreasureDistribution.values());
				comboBox.setBorder(BorderFactory.createEmptyBorder());
				comboBox.setBackground(new Color(255,255,210));
				comboBox.setSelectedItem(TreasureDistribution.fromValue(selVal.getValue()));
				comboBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						selVal.setValue(((TreasureDistribution) comboBox.getSelectedItem()).getValue());
						MainApp.getContent().changeHex();
					}
				});
				return comboBox;
			default:
				break;
		}
		return null;
	}

	public Object getCellEditorValue() {
		switch(selVal.getDataType()) {
			case SHORT:
				selVal.setValue(intBox.getValue().shortValue());
				MainApp.getContent().changeHex();
				return intBox.getValue().shortValue();
			case INTEGER:
				selVal.setValue(intBox.getValue());
				MainApp.getContent().changeHex();
				return intBox.getValue();
			case LONG:
				selVal.setValue(intBox.getValue_Long());
				MainApp.getContent().changeHex();
				return intBox.getValue_Long();
			case FLOAT:
				return selVal.getFloat();
			case STRING:
			case MESH_RES_STRING:
			case ANIM_RES_STRING:
			case LOC_STRING:
				selVal.setStringValue(textBox.getText());
				MainApp.getContent().changeHex();
				SwingUtilities.invokeLater(new Runnable() { public void run() { MainApp.getContent().fillTableData(true); } });
				return textBox.getText();
			case UNKNOWN_ENUM:
				selVal.setEnumInt(intBox.getValue());
				MainApp.getContent().changeHex();
				return intBox.getValue();
			case BOOLEAN:
			case ITEM_CATEGORY:
			case ATTRIB_REQ_OPERATION:
			case ATTRIB_MOD_OPERATION:
			case FOCUS_PRIORITY:
			case USE_TYPE:
			case FOCUS_NAME_TYPE:
			case ACTION:
			case AI_MODE:
			case AMBIENT_ACTION:
			case ANI_STATE:
			case ATTACK_REASON:
			case ATTITUDE:
			case BEARING:
			case CLASS:
			case DAMAGE_TYPE:
			case FOCUS:
			case FOCUS_LOOK_AT:
			case FOCUS_LOOK_AT_KEYS_FOR:
			case GENDER:
			case GUARD_STATUS:
			case HIT_DIRECTION:
			case LAST_FIGHT_AGAINST_PLAYER:
			case NPC_TYPE:
			case PARTY_MEMBER_TYPE:
			case PLAYER_CRIME:
			case POLITICAL_ALIGNMENT:
			case SLOT:
			case SPECIES:
			case TRADE_CATEGORY:
				
			case COLLISION_GROUP:
			case DIRECTIONAL_SHADOW_TYPE:
			case DYNAMIC_LIGHT_EFFECT:
			case EVENT_TYPE:
			case FINAL_BLEND:
			case LIGHTMAP_AMBIENT_OCCLUSION:
			case LIGHTMAP_TYPE:
			case PHYSIC_RANGE_TYPE:
			case PROPERTY_SET_TYPE:
			case SHAPE_GROUP:
			case SHAPE_MATERIAL:
			case STATIC_ILLUMINATED:
			case STATIC_LIGHING_TYPE:
			case STRIP_SPAWNING:
			case CAST_TYPE:
			case ENCLAVE_STATUS:
			case SKILL_CATEGORY:
			case SPELL_DEITY:
			case SPELL_TARGET:
			case SPELL_TYPE:
			case TREASURE_DISTRIBUTION:
				return comboBox.getSelectedItem();
			default:
				return null;
		}
	}
}
