package gui;

/**
 * 
 * Copyright 2007 - Dirk Becker, Germany
 * 
 * This Code ist FreeWare and OpenSource, feel free to Copy or Modify it!
 * 
 *  Commercial use is absolutely forbidden!
 *  This entail the use by someone who act for free in order of an commercial oriented company.
 *  Please stick to this rules!
 * 
 */
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.TableCellRenderer;

public class CustomTableCellRenderer implements TableCellRenderer {

	protected int length;
    protected char[] chars;
	protected ArrayList<Integer> COLUMNS;
	
	protected boolean isColored = false;
	protected boolean isMarked  = false;
	protected Color FirstColor  = new Color(255,255,245);
	protected Color SecondColor = new Color(255,252,255);

	protected void instance() { COLUMNS = new ArrayList<Integer>(); }

	/**
	 * <h1>CustomTableCellRenderer()</h1>
	 * Instanziiert einen TableCellRenderer mit erweiterten Eigenschaften.
	 */
	public CustomTableCellRenderer() {
		instance();
	}
	/**
	 * <h1>CustomTableCellRenderer(int[] ausrichtung)</h1>
	 * Instanziiert einen TableCellRenderer mit erweiterten Eigenschaften.
	 * @param ausrichtung bestimmt die Ausrichtung innerhalb der Zelle. Verwendet werden können die statischen SwingConstants-Attribute.
	 */
	public CustomTableCellRenderer(Integer[] ausrichtung) {
		instance();
		for(int it=0; it < ausrichtung.length; it++) {
			COLUMNS.add(ausrichtung[it]);
		}
	}

	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
		JLabel label = null;

		if(!(value instanceof JLabel) && value!=null) label = new JLabel(value.toString());
		else if(value!=null) label = (JLabel)value;
		else label = new JLabel();

		label.setOpaque(true);
		label.setFont(table.getFont());
		label.setForeground(table.getForeground());
		label.setBackground(table.getBackground());
		label.setPreferredSize(new Dimension(label.getPreferredSize().width, 20));
		label.setMaximumSize(new Dimension(label.getPreferredSize().width, 20));
		label.setMinimumSize(new Dimension(label.getPreferredSize().width, 20));
		label.setBorder(BorderFactory.createEmptyBorder(0,2,0,2));
		
		// Tooltip falls Eintrag nicht reinpasst
		chars = value.toString().toCharArray();
		length = label.getFontMetrics(label.getFont()).charsWidth(chars, 0, chars.length);
		if (table.getColumnModel().getColumn(column).getWidth() < length) label.setToolTipText(value.toString());
		else label.setToolTipText(null);

		// + * + * + * + * * + * + * + * + //
		/** Farbe von makierten Einträgen **/
		// + * + * + * + * * + * + * + * + //
		if(isSelected && label!=null) {
			label.setBackground(new Color(255,255,210));
			label.setForeground(table.getSelectionForeground());
		}
		else {
			// Hintergrundfarbe
			if(isColored && label !=null) {
				if(column%2==0) label.setBackground(FirstColor);
				else label.setBackground(SecondColor);
			}
		}

		if(label.getText().equals("use Hex-Editor")) {
			label.setFont(label.getFont().deriveFont(Font.ITALIC));
			label.setForeground(Color.GRAY);
		}


		// Ausrichten des Zellinhaltes
		while(COLUMNS.size()<=column) COLUMNS.add(SwingConstants.CENTER);
		if(label!=null) alignLabel(label, COLUMNS.get(column));

		// Icons
		if(value != null && value.toString().startsWith("setIcon:")) {
			try {
				label.setText(null);
				label.setToolTipText(null);
				label.setIcon(new ImageIcon(new URL(value.toString().substring(8))));
			}
			catch (MalformedURLException e) { e.printStackTrace(); }
		}

		return label;
	}
	
	protected void alignLabel(JLabel label, int align) {
		switch(align) {
			case SwingConstants.NORTH_WEST:
				label.setVerticalAlignment(SwingConstants.TOP);
				label.setHorizontalAlignment(SwingConstants.LEFT);
				break;
			case SwingConstants.NORTH:
				label.setVerticalAlignment(SwingConstants.TOP);
				label.setHorizontalAlignment(SwingConstants.CENTER);
				break;
			case SwingConstants.NORTH_EAST:
				label.setVerticalAlignment(SwingConstants.TOP);
				label.setHorizontalAlignment(SwingConstants.RIGHT);
				break;
			case SwingConstants.EAST:
				label.setVerticalAlignment(SwingConstants.CENTER);
				label.setHorizontalAlignment(SwingConstants.RIGHT);
				break;
			case SwingConstants.SOUTH_EAST:
				label.setVerticalAlignment(SwingConstants.BOTTOM);
				label.setHorizontalAlignment(SwingConstants.RIGHT);
				break;
			case SwingConstants.SOUTH:
				label.setVerticalAlignment(SwingConstants.BOTTOM);
				label.setHorizontalAlignment(SwingConstants.CENTER);
				break;
			case SwingConstants.SOUTH_WEST:
				label.setVerticalAlignment(SwingConstants.BOTTOM);
				label.setHorizontalAlignment(SwingConstants.LEFT);
				break;
			case SwingConstants.WEST:
				label.setVerticalAlignment(SwingConstants.CENTER);
				label.setHorizontalAlignment(SwingConstants.LEFT);
				break;
			case SwingConstants.CENTER:
				label.setVerticalAlignment(SwingConstants.CENTER);
				label.setHorizontalAlignment(SwingConstants.CENTER);
				break;
		}
	}

	public Color getFirstColor() { return FirstColor; }
	public void setFirstColor(Color firstColor) { FirstColor = firstColor; }
	public Color getSecondColor() { return SecondColor; }
	public void setSecondColor(Color secondColor) { SecondColor = secondColor; }
	public boolean isColored() { return isColored; }
	public void setColored(boolean isColored) { this.isColored = isColored; }
	public boolean isMarked() { return isMarked; }
	public void setMarked(boolean isMarked) { this.isMarked = isMarked; }
}