package gui;

/**
 * 
 * Copyright 2007 - Dirk Becker, Germany
 * 
 * This Code ist FreeWare and OpenSource, feel free to Copy or Modify it!
 * 
 *  Commercial use is absolutely forbidden!
 *  This entail the use by someone who act for free in order of an commercial oriented company.
 *  Please stick to this rules!
 * 
 */
import java.awt.Color;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.DecimalFormat;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JTextField;

public class IntegerField extends JTextField {
	
	private static final long serialVersionUID = 2240515324034560146L;
	private boolean mechanicalChange = false;
	private boolean badCommata = false;
	
	public IntegerField() {
		super();
		setBorder(BorderFactory.createEmptyBorder());
		setHorizontalAlignment(JLabel.RIGHT);
		setText("0");
		this.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent e) {
				if(e.getKeyCode()==110 && badCommata==false) {
					e.consume();
					badCommata = true;
				}
			}
			public void keyReleased(KeyEvent e) {
				if(e.getKeyCode()==110) {
					e.consume();
				}
			}
			public void keyTyped(KeyEvent e) {
				if(badCommata) {
					e.consume();
					badCommata = false;
					int CaretPosition = getCaretPosition();
					setText(getText().substring(0, CaretPosition) + "," + getText().substring(CaretPosition));
					setCaretPosition(CaretPosition+1);
				}
			}
		});
		this.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e)  {}
			public void mouseEntered(MouseEvent e)  {}
			public void mouseExited(MouseEvent e)   {}
			public void mousePressed(MouseEvent e)  { showPopUp(e); }
			public void mouseReleased(MouseEvent e)  { showPopUp(e); }
		});
		this.addFocusListener(new FocusListener(){
			public void focusGained(FocusEvent e) {
				mechanicalChange = true;
				if(isEditable()) {
					setBackground(new Color(255,255,210));
					setText(getText());
					selectAll();
					FocusGainedExtra();
				}
				mechanicalChange = false;
			}
			public void focusLost(FocusEvent evt) {
				mechanicalChange = true;
				if(isEditable()) {
					setBackground(Color.WHITE);
					formatIntegerField();
					FocusLostExtra();
				}
				mechanicalChange = false;
			}
		});
	}

	public Integer getValue() {
		String tempText = getText();
		if(tempText.equals("")) tempText = "0";
		char[] TextArray = tempText.trim().toCharArray();
		tempText = "";
		for(int i=0; i<TextArray.length; i++) {
			if(TextArray[i]!='.') {
				if(TextArray[i]==',') tempText += ".";
				else tempText += TextArray[i];
			}
		}
		Double d = Double.parseDouble(tempText);
		return d.intValue();
	}

	public Long getValue_Long() {
		String tempText = getText();
		if(tempText.equals("")) tempText = "0";
		char[] TextArray = tempText.trim().toCharArray();
		tempText = "";
		for(int i=0; i<TextArray.length; i++) {
			if(TextArray[i]!='.') {
				if(TextArray[i]==',') tempText += ".";
				else tempText += TextArray[i];
			}
		}
		Double d = Double.parseDouble(tempText);
		return d.longValue();
	}

	public void setValue(Integer mValue) {
		mechanicalChange = true;
		setText(mValue.toString());
		formatIntegerField();
		mechanicalChange = false;
	}

	public void setValue(Long mValue) {
		mechanicalChange = true;
		setText(mValue.toString());
		formatIntegerField();
		mechanicalChange = false;
	}

	private void formatIntegerField() {
		String content = getText();
		if (content.length() != 0) {
			// Überprüfe ob Wert OK ist und bei Bedarf abändern oder warnen
			try {
				Double d = Double.parseDouble(content.replace(',','.'));
				DecimalFormat df = new DecimalFormat("###########0");
			    setText(df.format(d.longValue()));
			    setBorder(BorderFactory.createEmptyBorder());
			} catch (NumberFormatException nfe) {
				getToolkit().beep();
				requestFocus();
				setBorder(BorderFactory.createLineBorder(new java.awt.Color(255,100,50),1));
			}
		}
		else {
			setText("0");
			setBorder(BorderFactory.createEmptyBorder());
		}
	}

	
	public void FocusLostExtra() {
		// Überschreiben für Extra Focus-Funktionen
	}
	
	public void FocusGainedExtra() {
		// Überschreiben für Extra Focus-Funktionen
	}			

	private void showPopUp(MouseEvent e) {
		if(e.isPopupTrigger() && this.isEnabled()) {
			this.requestFocus();
		}
	}
	
	public void setEnabled(Boolean enabled) { super.setEnabled(enabled); }

	public boolean isMechanicalChange() { return mechanicalChange; }
}
