package gui;

/**
 * 
 * Copyright 2007 - Dirk Becker, Germany
 * 
 * This Code ist FreeWare and OpenSource, feel free to Copy or Modify it!
 * 
 *  Commercial use is absolutely forbidden!
 *  This entail the use by someone who act for free in order of an commercial oriented company.
 *  Please stick to this rules!
 * 
 */
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import main.TPLE_Editor;
import main.data.TPLE_File;

public class MainMenu extends JMenuBar {

	private static final long serialVersionUID = -4018906463631459057L;

	private String currentFile = "";
	private TPLE_Editor MainApp;
	private JMenu mnuFile;
	private JMenuItem mnuFileOpen;
	private JMenuItem mnuFileSave;
	private JMenuItem mnuFileSaveAs;
	private JMenuItem mnuFileClose;
	private JMenuItem mnuFileExit;
	private JMenu mnuPref;
	private JCheckBoxMenuItem mnuPrefHexVisible;
	private JCheckBoxMenuItem mnuPrefCheckUpdate;
	private JMenu mnuInfo;
	private JMenuItem mnuAbout;
	private JMenu mnuUpdate;
	private JMenuItem mnuUpdCheck;

	public MainMenu(final TPLE_Editor app) {
		MainApp = app;
		mnuFile = new JMenu();
		mnuFile.setText("File");
		add(mnuFile);

		mnuFileOpen = new JMenuItem();
		mnuFileOpen.setText("Open");
		mnuFileOpen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mnuFileOpen.isEnabled()) {
					File f = MainApp.getFileLoadLocation();
					if(f!=null) {
						MainApp.setCurrentFile(new TPLE_File(f));
						MainApp.refreshTreeContent();
						MainApp.getContent().getTreeView().expandAll(true);
						MainApp.getContent().getTreeView().setSelectionRow(0);
						currentFile = f.getName() + " - ";
						mnuFileSave.setEnabled(true);
						mnuFileSaveAs.setEnabled(true);
						mnuFileClose.setEnabled(true);
						MainApp.setTitle(currentFile + "G3 Template Editor - Not for commercial use !!");
					}
				}
			}
		});
		mnuFile.add(mnuFileOpen);

		mnuFileSave = new JMenuItem();
		mnuFileSave.setText("Save");
		mnuFileSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mnuFileSave.isEnabled()) {
					MainApp.getCurrentFile().save();
				}
			}
		});
		mnuFileSave.setEnabled(false);
		mnuFile.add(mnuFileSave);

		mnuFileSaveAs = new JMenuItem();
		mnuFileSaveAs.setText("Save as ..");
		mnuFileSaveAs.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mnuFileSaveAs.isEnabled()) {
					File f = MainApp.getFileSaveLocation();
					if(f!=null) MainApp.getCurrentFile().saveAs(f);
				}
			}
		});
		mnuFileSaveAs.setEnabled(false);
		mnuFile.add(mnuFileSaveAs);

		mnuFileClose = new JMenuItem();
		mnuFileClose.setText("Close");
		mnuFileClose.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mnuFileClose.isEnabled()) {
					MainApp.setCurrentFile(null);
					MainApp.refreshTreeContent();
					MainApp.getContent().clearHexFields();
					currentFile = "";
					mnuFileSave.setEnabled(false);
					mnuFileSaveAs.setEnabled(false);
					mnuFileClose.setEnabled(false);
					MainApp.setTitle("G3 Template Editor - Not for commercial use !!");
				}
			}
		});
		mnuFileClose.setEnabled(false);
		mnuFile.add(mnuFileClose);

		mnuFile.add(new JSeparator());

		mnuFileExit = new JMenuItem();
		mnuFileExit.setText("Exit");
		mnuFileExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mnuFileExit.isEnabled()) {
					System.exit(0);
				}
			}
		});
		mnuFile.add(mnuFileExit);

		mnuPref = new JMenu();
		mnuPref.setText("Preferences");
		add(mnuPref);
		
		mnuPrefHexVisible = new JCheckBoxMenuItem();
		mnuPrefHexVisible.setText("Show Hexeditor");
		mnuPrefHexVisible.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mnuPrefHexVisible.isEnabled()) {
					MainApp.getContent().setHexeditorVisible(mnuPrefHexVisible.isSelected());
					MainApp.getOptions().setHexVisible(mnuPrefHexVisible.isSelected());
				}
			}
		});
		mnuPref.add(mnuPrefHexVisible);

		mnuPrefCheckUpdate = new JCheckBoxMenuItem();
		mnuPrefCheckUpdate.setText("check Version on start");
		mnuPrefCheckUpdate.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mnuPrefCheckUpdate.isEnabled()) {
					MainApp.getOptions().setCheckUpdate(mnuPrefCheckUpdate.isSelected());
				}
			}
		});
		mnuPref.add(mnuPrefCheckUpdate);


		mnuInfo = new JMenu();
		mnuInfo.setText("Info");
		add(mnuInfo);

		mnuAbout = new JMenuItem();
		mnuAbout.setText("About");
		mnuAbout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mnuAbout.isEnabled()) {
					JPanel p = new JPanel(new GridLayout(0, 1));
					JLabel version = new JLabel("G3 Template Editor - V." + TPLE_Editor.VERSION);
					version.setFont(new Font("Verdana", Font.BOLD, 10));
					p.add(version);

					JLabel copy = new JLabel(" 2007 - Becker Print Soft (DE)");
					copy.setFont(new Font("Verdana", Font.PLAIN, 8));
					p.add(copy);

					JOptionPane.showMessageDialog(null, p, "About G3 Template Editor", JOptionPane.PLAIN_MESSAGE);
					
				}
			}
		});
		mnuInfo.add(mnuAbout);
		
		add(Box.createGlue());
		
		mnuUpdate = new JMenu();
		mnuUpdate.setIcon(new ImageIcon(MainMenu.class.getResource("images/updated.png")));
		mnuUpdate.setText("Update");
		add(mnuUpdate);

		mnuUpdCheck = new JMenuItem();
		mnuUpdCheck.setText("Check Now!");
		mnuUpdCheck.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(mnuUpdCheck.isEnabled()) {
					if(!MainApp.getOptions().checkVersion()) {
						JOptionPane.showMessageDialog(	null,
							"There is no Update available at this moment!",
							"No Update available!",
							JOptionPane.INFORMATION_MESSAGE,
							new ImageIcon(MainMenu.class.getResource("images/updnfod.png")) );
					}
				}
			}
		});
		mnuUpdate.add(mnuUpdCheck);
	}
	
	public void newUpdateAviable() {
		mnuUpdate.setIcon(new ImageIcon(MainMenu.class.getResource("images/update.png")));
	}
	
	public void setHexVisSelected(boolean selected) { mnuPrefHexVisible.setSelected(selected); }
	public void setUpdateSelected(boolean selected) { mnuPrefCheckUpdate.setSelected(selected); }
}
