package gui;

/**
 * 
 * Copyright 2007 - Dirk Becker, Germany
 * 
 * This Code ist FreeWare and OpenSource, feel free to Copy or Modify it!
 * 
 *  Commercial use is absolutely forbidden!
 *  This entail the use by someone who act for free in order of an commercial oriented company.
 *  Please stick to this rules!
 * 
 */
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Vector;
import javax.swing.table.DefaultTableModel;

public class SortableTableModel extends DefaultTableModel {
	
	private static final long serialVersionUID = 5690973161985894571L;
	private int lastSelectedColumn = -1;
	public boolean[] sortColumnDesc;

	public SortableTableModel(Object[][] rowData, Object[] headers) {
		super(rowData, headers);
		sortColumnDesc = new boolean[headers.length];
	}

	@SuppressWarnings("unchecked")
	public void sortByColumn(final int clm) {
		lastSelectedColumn = clm;
		Collections.sort(this.dataVector, new Comparator() {
			public int compare(Object o1, Object o2) {
				
				Vector v1 = (Vector) o1;
				Vector v2 = (Vector) o2;

				int size1 = v1.size();
				if (clm >= size1)
					throw new IllegalArgumentException("max column idx: " + size1);

				Comparable c1 = (Comparable) v1.get(clm);
				Comparable c2 = (Comparable) v2.get(clm);

				int cmp = 0;
				boolean isString = true;

				// test INTEGER
				try {
					Integer i1 = Integer.parseInt(c1.toString());
					Integer i2 = Integer.parseInt(c2.toString());
					cmp = i1.compareTo(i2);
					isString = false;
				}
				catch(NumberFormatException e) {}
				
				// test LONG
				try {
					Long l1 = Long.parseLong(c1.toString());
					Long l2 = Long.parseLong(c2.toString());
					cmp = l1.compareTo(l2);
					isString = false;
				}
				catch(NumberFormatException e) {}
				
				// test DOUBLE
				try {
					Double d1 = Double.parseDouble(c1.toString());
					Double d2 = Double.parseDouble(c2.toString());
					cmp = d1.compareTo(d2);
					isString = false;
				}
				catch(NumberFormatException e) {}
				
				// test DATE
				if(c1 instanceof Date && c2 instanceof Date) {
					cmp = ((Date)c1).compareTo((Date)c2);
					isString = false;
				}

				// compare as STRING
				if(isString) cmp = prepairForCompare(c1).compareTo(prepairForCompare(c2));

				if (sortColumnDesc[clm]) cmp *= -1;

				return cmp;
			}
		});
		this.sortColumnDesc[clm] ^= true;
	}
	
	private String prepairForCompare( Object o ) {
		return o.toString().toLowerCase().replace( '', 'a' ).replace( '', 'o' ).replace( '', 'u' ).replace( '', 's' );
	}

	public int getLastSelectedColumn() { return lastSelectedColumn; }
}
