package gui;

/**
 * 
 * Copyright 2007 - Dirk Becker, Germany
 * 
 * This Code ist FreeWare and OpenSource, feel free to Copy or Modify it!
 * 
 *  Commercial use is absolutely forbidden!
 *  This entail the use by someone who act for free in order of an commercial oriented company.
 *  Please stick to this rules!
 * 
 */
import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicTreeUI;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

public class TreeView extends JTree {
	private static final long serialVersionUID = -4353855249576315684L;
	private BPSTReeRenderer renderer = new BPSTReeRenderer();

	public TreeView() {
		setTreeOpenIcon(new ImageIcon(TreeView.class.getResource("images/package.png")));
		setTreeClosedIcon(new ImageIcon(TreeView.class.getResource("images/package.png")));
		setTreeLeafIcon(new ImageIcon(TreeView.class.getResource("images/class.png")));
		setTreeCollapsedIcon(new ImageIcon(TreeView.class.getResource("images/tree_plus.png")));
		setTreeExpandedIcon(new ImageIcon(TreeView.class.getResource("images/tree_minus.png")));
		setCellRenderer(renderer);
	}
	
	public void setTreeLeafIcon(ImageIcon ico)			{ renderer.DefaultItemIcon = ico; }
	public void setTreeOpenIcon(ImageIcon ico)			{ renderer.OpenContainerIcon = ico; }
	public void setTreeClosedIcon(ImageIcon ico)		{ renderer.ClosedContainerIcon = ico; }
	public void setTreeCollapsedIcon(ImageIcon ico)	{ ((BasicTreeUI)getUI()).setCollapsedIcon(ico); }
	public void setTreeExpandedIcon(ImageIcon ico)		{ ((BasicTreeUI)getUI()).setExpandedIcon(ico); }
	public void setItemBorder(Border b)				{ renderer.ItemBorder = b; }
	public void setSelectedItemBorder(Border b)		{ renderer.SelectedItemBorder = b; }
	public void setForegroundColor(Color c)			{ renderer.ForegroundColor = c; }
	public void setBackgroundColor(Color c)			{ renderer.BackgroundColor = c; }
	public void setSelectionForegroundColor(Color c)	{ renderer.SelectionForegroundColor = c; }
	public void setSelectionBackgroundColor(Color c)	{ renderer.SelectionBackgroundColor = c; }

	private class BPSTReeRenderer extends JLabel implements TreeCellRenderer {
		protected static final long serialVersionUID		= -1071037319496976578L;
		protected JLabel			itemLabel					= new JLabel();
		protected ImageIcon		OpenContainerIcon			= null;
		protected ImageIcon		ClosedContainerIcon			= null;
		protected ImageIcon		DefaultItemIcon				= null;
		protected Border			ItemBorder					= BorderFactory.createEmptyBorder(3,3,3,3);
		protected Border			SelectedItemBorder			= BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(new Color(230, 238, 248)), BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(new Color(184, 207, 229)), BorderFactory.createLineBorder(new Color(230, 238, 248))));
		protected Color			ForegroundColor				= Color.BLACK;
		protected Color			BackgroundColor				= Color.WHITE;
		protected Color			SelectionForegroundColor	= Color.BLACK;
		protected Color			SelectionBackgroundColor	= new Color(223, 235, 245);

		public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
	    	DefaultMutableTreeNode currentItem = (DefaultMutableTreeNode) value;

	    	if(expanded)	itemLabel.setIcon(currentItem.isLeaf() ? DefaultItemIcon : OpenContainerIcon);
	    	else			itemLabel.setIcon(currentItem.isLeaf() ? DefaultItemIcon : ClosedContainerIcon);

	    	itemLabel.setText(value.toString());
	    	itemLabel.setFont(tree.getFont());
	    	itemLabel.setOpaque(true);

	    	if(selected) {
	    		itemLabel.setForeground(SelectionForegroundColor);
	    		itemLabel.setBackground(SelectionBackgroundColor);
	    		itemLabel.setBorder(SelectedItemBorder);
	    	}
	    	else {
	    		itemLabel.setForeground(ForegroundColor);
	    		itemLabel.setBackground(BackgroundColor);
	    		itemLabel.setBorder(ItemBorder);
	    	}

	    	getAdvancedIcon(itemLabel, currentItem, selected, expanded, row, hasFocus);

	    	return itemLabel;
	     }

		public void getAdvancedIcon(JLabel currentItem, DefaultMutableTreeNode node, boolean selected, boolean expanded, int row, boolean hasFocus) {}
	}

	public DefaultMutableTreeNode getRoot() { return (DefaultMutableTreeNode)getModel().getRoot(); }
	public DefaultTreeModel getTreeModel() { return (DefaultTreeModel)getModel(); }

	public void insertNewChild(DefaultMutableTreeNode node) { insertNewChild(node,getRoot(),true); }
	public void insertNewChild(DefaultMutableTreeNode node, DefaultMutableTreeNode parent) { insertNewChild(node, parent, true); }
	public void insertNewChild(DefaultMutableTreeNode node, DefaultMutableTreeNode parent, boolean expnsel) {
		int position = parent.equals(getRoot()) ? 1 : 0; 
		getTreeModel().insertNodeInto(node, parent, position);
		getTreeModel().nodeStructureChanged(parent);
		if(expnsel) {
			expandTo(node, true);
			selectNode(node);
		}
	}

	public void removeSelectedChild() {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)getLastSelectedPathComponent();
		if(node!=null) removeChild(node);
	}
	public void removeChild(DefaultMutableTreeNode node) { removeChild(node,getRoot(),true); }
	public void removeChild(DefaultMutableTreeNode node, DefaultMutableTreeNode parent) { removeChild(node, parent, true); }
	public void removeChild(DefaultMutableTreeNode node, DefaultMutableTreeNode parent, boolean expnsel) {
		getTreeModel().removeNodeFromParent(node);
		getTreeModel().nodeStructureChanged(parent);
		if(expnsel) {
			expandTo(node, true);
			selectNode(node);
		}
	}

	public void selectNode(TreeNode node) {
		TreeNode parent = node;
		ArrayList<TreeNode> nodes = new ArrayList<TreeNode>();
		nodes.add(node);
		while((parent=parent.getParent())!=null) nodes.add(0, parent);
		setSelectionPath(new TreePath(nodes.toArray(new Object[0])));
	}

	public void expandTo(TreeNode node, boolean expand) {
		TreeNode parent = node;
		ArrayList<TreeNode> nodes = new ArrayList<TreeNode>();
		nodes.add(node);
		while((parent=parent.getParent())!=null) nodes.add(0, parent);
		while(nodes.size()>0) {
	        if(expand) expandPath(new TreePath(nodes.toArray(new Object[0])));
	        else collapsePath(new TreePath(nodes.toArray(new Object[0])));
	        nodes.remove(nodes.size()-1);
		}
    }

	public void expandAll(boolean expand) { expandAll(new TreePath(getRoot()), expand); }
	public void expandAll(TreePath parent, boolean expand) {
        TreeNode node = (TreeNode)parent.getLastPathComponent();
        if (node.getChildCount() >= 0) {
            for (Enumeration e=node.children(); e.hasMoreElements(); ) {
                TreeNode n = (TreeNode)e.nextElement();
                TreePath path = parent.pathByAddingChild(n);
                expandAll(path, expand);
            }
        }
        if(expand) expandPath(parent);
        else collapsePath(parent);
    }
}
