package main;

/**
 * 
 * Copyright 2007 - Dirk Becker, Germany
 * 
 * This Code ist FreeWare and OpenSource, feel free to Copy or Modify it!
 * 
 *  Commercial use is absolutely forbidden!
 *  This entail the use by someone who act for free in order of an commercial oriented company.
 *  Please stick to this rules!
 * 
 */
import java.awt.BorderLayout;
import java.io.File;
import java.io.FilenameFilter;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.filechooser.FileFilter;
import opt.Options;
import main.data.Converter;
import main.data.TPLE_Class;
import main.data.TPLE_File;
import gui.ContentPane;
import gui.MainMenu;
import gui.StatusBar;

public class TPLE_Editor extends JFrame {

	public static final String VERSION = "0.04";
	private static final long serialVersionUID = -8447120230885011626L;

	static final String It0 = "C:\\G3_TPLE\\_Weapons_1H_Story_It_1H_FlameSword_01.tple";	// 477 / 481
	static final String It1 = "C:\\G3_TPLE\\_Weapons_1H_World_It_1H_Sword_Rust_01.tple";
	static final String It2 = "C:\\G3_TPLE\\Mission_Items_Story_It_Enzo_Supply.tple";
	static final String It3 = "C:\\G3_TPLE\\Mission_Items_Story_It_Giores_Supply.tple";
	static final String It4 = "C:\\G3_TPLE\\Adanos_Artefacts_Story_It_Artefact_Ring_01.tple";
	static final String It5 = "C:\\G3_TPLE\\Action_Items.lrtpl";

	private MainMenu		mainMenu;
	private Options		options;
	private ContentPane	contentPane;
	private StatusBar		statusBar;
	public	 File			FileSaveLocation = null;
	public	 File			FileLoadLocation = null;
	private TPLE_File		currentFile		 = null;
	private TPLE_Class		selectedClass	 = null;
	
	public TPLE_Editor() {
		setTitle("G3 Template Editor - Not for commercial use !!");
		setSystemUI();
		setSize(640, 460);
		setResizable(true);
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		System.out.println("Hallo");

		mainMenu = new MainMenu(this);
		setJMenuBar(mainMenu);

		contentPane = new ContentPane(this);
		add(contentPane, BorderLayout.CENTER);

		statusBar = new StatusBar();
		add(statusBar, BorderLayout.SOUTH);

		setLocationRelativeTo(null);
		refreshTreeContent();

		options = new Options();
		getContent().setHexeditorVisible(options.isHexVisible());
		mainMenu.setHexVisSelected(options.isHexVisible());
		mainMenu.setUpdateSelected(options.canDoUpdate());
		FileLoadLocation = options.getLoadPath();
		FileSaveLocation = options.getSavePath();
		
		setVisible(true);

		if(options.canDoUpdate() && !options.wasNotified()) options.checkVersion();
		if(options.doesNewExists()) mainMenu.newUpdateAviable();
	}

	public static void main(String[] args) { new TPLE_Editor(); }
	
	private void setSystemUI() {
		// setting LookAndFeel
		try { UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName()); }
		catch(Exception e) {
			try { UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName()); } 
			catch(Exception e2) { e2.printStackTrace(); }
		}
	}
	
	public Options getOptions() { return options; }

	public void refreshTreeContent() {
		if(currentFile!=null) {
			contentPane.setTreeModel(currentFile.getTreeModel());
			getContent().setItemHash();
			getStatusBar().setStatusText("<html><body><strong>Item-Hash:</strong> " +
					Converter.byteToHexString(getCurrentFile().getItemHash()) +
					"<br /><strong>Haendler-Hash:</strong> " +
					Converter.byteToHexString(getCurrentFile().getHaendlerHash()) + "</body></html>");
		}
		else contentPane.setTreeModel(null);
		setSelectedClass(null);
		getContent().fillTableData();
		repaint();
	}

	public ContentPane getContent() { return contentPane; }

	public File getFileSaveLocation() { return getFileSaveLocation(null); }
	public File getFileSaveLocation(String filename) {
		try {
			JFileChooser d = new JFileChooser();
			d.setCurrentDirectory(FileSaveLocation);
			if(filename!=null) d.setSelectedFile(new File(FileSaveLocation+filename));

			d.setFileSelectionMode(JFileChooser.FILES_ONLY); 
			int returnVal = d.showSaveDialog(this);
	        if(returnVal==JFileChooser.APPROVE_OPTION) FileSaveLocation = d.getSelectedFile().getParentFile();
	        else return null;
	        options.setSaveLocation(FileSaveLocation.getParent());
	        return d.getSelectedFile();
		}
		catch(Exception ex) { ex.printStackTrace(); return null; }
	}

	public File getFileLoadLocation() { return getFileLoadLocation(new String[]{}); }
	public File getFileLoadLocation(final String[] extensions) { return getLoadLocation(extensions, false); }
	public File getFolderLoadLocation() { return getLoadLocation(new String[]{}, true); }
	private File getLoadLocation(final String[] extensions, boolean folderbrowsing) {
		try {
			JFileChooser d = new JFileChooser();
			d.setCurrentDirectory(FileLoadLocation);
			if(extensions.length>0) {
				d.setFileFilter(new FileFilter() {
					public boolean accept(File f) {
						if(f.isDirectory()) return true;
						for(String ext: extensions) if (f.getName().endsWith(ext)) return true;
						return false;
					}
					public String getDescription() { return null; }
				});
			}
			d.setFileSelectionMode(folderbrowsing ? JFileChooser.DIRECTORIES_ONLY : JFileChooser.FILES_ONLY); 
			int returnVal = d.showOpenDialog(this);
	        if(returnVal==JFileChooser.APPROVE_OPTION) FileLoadLocation = d.getSelectedFile();
	        else return null;
	        options.setLoadLocation(FileLoadLocation.getParent());
	        return FileLoadLocation;
		}
		catch(Exception ex) { ex.printStackTrace(); return null; }
	}
	public FilenameFilter getDefaultFileFilter() {
		return new FilenameFilter() {
			public boolean accept(File dir, String name) {
				if( name.toLowerCase().startsWith("datanorm.") ||
					name.toLowerCase().startsWith("datasets.") ||
					name.toLowerCase().startsWith("datatext.") ||
					name.toLowerCase().startsWith("datpreis.") ||
					name.toLowerCase().startsWith("datainfo.") ||
					name.toLowerCase().endsWith(".rab")   ||
					name.toLowerCase().endsWith(".wrg")) return true;
				return false;
			}
		};
	}

	public TPLE_File getCurrentFile() { return currentFile; }
	public void setCurrentFile(TPLE_File currentFile) { this.currentFile = currentFile; }

	public TPLE_Class getSelectedClass() { return selectedClass; }
	public void setSelectedClass(TPLE_Class selectedClass) { this.selectedClass = selectedClass; }

	public StatusBar getStatusBar() { return statusBar; }
}
