package main.data;

/**
 * 
 * Copyright 2007 - Dirk Becker, Germany
 * 
 * This Code ist FreeWare and OpenSource, feel free to Copy or Modify it!
 * 
 *  Commercial use is absolutely forbidden!
 *  This entail the use by someone who act for free in order of an commercial oriented company.
 *  Please stick to this rules!
 * 
 */
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.List;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;

public class TPLE_File {

	private File				orgFile		= null;
	private TPLE_String		filename	= new TPLE_String();
	private TPLE_String		itemName	= new TPLE_String();
	private byte[]				itemHash	= new byte[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	private long				hashOffset	= 0L;
	private byte[]				haendlHash	= new byte[] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	private long				hhashOffset	= 0L;
	private List<TPLE_Class>	classes		= new ArrayList<TPLE_Class>();
	

	private long	ENDE = 0;
	private byte[]	Bit16 = new byte[2];
	private byte[]	Bit32 = new byte[4];
	private byte[]	Bit128 = new byte[16];
	private StringPointer strings = new StringPointer(); 
              
	private RandomAccessFile raf;
	private boolean firstClass = true;

	public TPLE_File(File file) {
		try {
			orgFile = file;
			raf = new RandomAccessFile(file, "r");
			readStructure();
			readData();
		}
		catch(Exception e) { e.printStackTrace(); }
	}

	void readData() {
		try {
			setFilename(strings.get(0));
			setItemName(strings.get(1));

			raf.seek(10);
			raf.read(Bit32);
			ENDE = Converter.UInt32ToLong(Bit32)-20;

			raf.seek(27);
			raf.read(Bit32);
			long length = Converter.UInt32ToLong(Bit32);
			raf.seek(raf.getFilePointer()+(length*2)+12);

			// Item-Hash
			setHashOffset(raf.getFilePointer());
			raf.read(Bit128);
			setItemHash(Bit128.clone());

			// Springe zum Haendler-Hash, 251 fix ...
			raf.seek(raf.getFilePointer()+251);
			setHaendlerHashOffset(raf.getFilePointer());
			raf.read(Bit128);
			setHaendlerHash(Bit128.clone());
			
			// Springe zur ersten Klasse, 253 fix ...
			raf.seek(raf.getFilePointer()+253);
			
			TPLE_Class tpleClass = null;
			while((tpleClass=getNextClass())!=null) addClass(tpleClass);
		}
		catch(Exception e) { e.printStackTrace(); }
	}
	
	private TPLE_Class getNextClass() {
		TPLE_Class newClass = null;
		try {
			if(!firstClass) {
				boolean search = true;
				while(search) {
					if(raf.getFilePointer()>=ENDE) return null;

					// 222, 192, 173
					if(raf.readUnsignedByte()==222)
						if(raf.readUnsignedByte()==192)
							if(raf.readUnsignedByte()==173) {
								search = false;
								raf.seek(raf.getFilePointer()+7);
							}
				}
			}
			else firstClass = false;

			newClass = new TPLE_Class(this);

			raf.read(Bit16);
			newClass.setClassName(strings.get(Converter.UInt16ToLong(Bit16)));

//			System.out.println("class " + newClass.getClassName().getValue() + " {");

			raf.seek(raf.getFilePointer()+21);
			readValues(newClass);

//			System.out.println("}\n");
		}
		catch(Exception e) { e.printStackTrace(); }
		return newClass;
	}
	
	private void readValues(TPLE_Class newClass) {
		try {
			raf.read(Bit32);
			long values = Converter.UInt32ToLong(Bit32);
			
			for(;values>0; values--) {
				TPLE_Value value = new TPLE_Value(newClass);

				raf.read(Bit16);
				value.setName(strings.get(Converter.UInt16ToLong(Bit16)));

				raf.read(Bit16);
				value.setType(strings.get(Converter.UInt16ToLong(Bit16)));
				
				// 1E 00 - Marke berspringen
				raf.seek(raf.getFilePointer()+2);

				raf.read(Bit32);
				value.setPosition(raf.getFilePointer());
				byte[] b = new byte[(int)Converter.UInt32ToLong(Bit32)];
				raf.read(b);
				value.setValue(b);
				if(value.isString()) value.setString(strings.get(value.getShort()));
				
//				System.out.println("\t" + value.getType().getValue() + " " + value.getName().getValue());

				newClass.addValue(value);
			}
		}
		catch(Exception e) { e.printStackTrace(); }
	}

	void readStructure() {
		try {
			raf.seek(10);
			raf.read(Bit32);
			long point = Converter.UInt32ToLong(Bit32);
			raf.seek(point+9);
			
			Integer it = 0;
			while(raf.read(Bit16)>0) {
				TPLE_String value = new TPLE_String();
				value.setPosition(raf.getFilePointer()-2);
				value.setIndex(it);

				String temp = "";
				for(int c=0; c<Converter.UInt16ToLong(Bit16); c++)
					temp += (char)raf.readUnsignedByte();
				value.setValue(temp);

				strings.put((long)it, value);
				strings.setLastIndex(it++);
			}
		}
		catch(Exception e) { e.printStackTrace(); }
	}

	public TPLE_String getFilename() { return filename; }
	public void setFilename(TPLE_String filename) { this.filename = filename; }

	public byte[] getItemHash() { return itemHash; }
	public void setItemHash(byte[] itemHash) { if(itemHash.length==16) this.itemHash = itemHash; }

	public byte[] getHaendlerHash() { return haendlHash; }
	public void setHaendlerHash(byte[] haendlHash) { this.haendlHash = haendlHash; }

	public TPLE_String getItemName() { return itemName; }
	public void setItemName(TPLE_String itemName) { this.itemName = itemName; }

	public List<TPLE_Class> getClasses() { return classes; }
	public void setClasses(List<TPLE_Class> classes) { this.classes = classes; }
	public void addClass(TPLE_Class cl) { this.classes.add(cl); }
	public void removeClass(TPLE_Class cl) { this.classes.remove(cl); }

	public long getHashOffset() { return hashOffset; }
	public void setHashOffset(long hashOffset) { this.hashOffset = hashOffset; }

	public long getHaendlerHashOffset() { return hhashOffset; }
	public void setHaendlerHashOffset(long hashOffset) { this.hhashOffset = hashOffset; }
	
	public TreeModel getTreeModel() {
		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(this);

		for(TPLE_Class cl: classes) {
			DefaultMutableTreeNode entryNode = new DefaultMutableTreeNode(cl);
			rootNode.add(entryNode);
		}

		return new DefaultTreeModel(rootNode);
	}
	
	public String toString() { return itemName.getValue(); }
	
	public void newStringValue(int index, String string) {
		boolean firstOccur = true;
		for(TPLE_Class cl: classes) {
			for(TPLE_Value val: cl.getValues()) {
				if(val.isString() && val.getString().getIndex()==index) {
					val.getString().setValue(string);
					if(firstOccur) {
						strings.put((long)val.getString().getIndex(), val.getString());
						firstOccur = false;
					}
				}
			}
		}
	}
	
	public void save() {
		try {
			File orgClone = new File(orgFile.getAbsolutePath());
			File tempFile = File.createTempFile("g3ed_", ".tple");
			saveAs(tempFile);
			copy(tempFile, orgClone);
			tempFile.delete();
			orgFile = orgClone;
		}
		catch(IOException e) { e.printStackTrace(); }
	}
	public void saveAs(File target) {
		try {
			FileOutputStream out = new FileOutputStream(target);
			RandomAccessFile raf = new RandomAccessFile(orgFile, "r");

			raf.seek(10);
			raf.read(Bit32);
			int length = (int)Converter.UInt32ToLong(Bit32)+9;
			raf.seek(0);

			/**
			 *    old file, new strings
			 **/
			byte[] data = new byte[length];
			raf.read(data);
			out.write(data);

			// strings
			for(long index=0; index<=strings.getLastIndex(); index++) {
				String tmpString = strings.get(index).getValue();
				out.write(Converter.longToUInt16((long)tmpString.length()));
				out.write(tmpString.getBytes());
			}
			out.flush();
			out.close();
			raf.close();
			

			/**
			 *    update values
			 **/
			raf = new RandomAccessFile(target, "rw");
			// Item-Hash
			raf.seek(getHashOffset());
			raf.write(getItemHash());
			// Haendler-Hash
			raf.seek(getHaendlerHashOffset());
			raf.write(getHaendlerHash());
			
			// item values
			for(TPLE_Class cl: classes) {
				for(TPLE_Value value: cl.getValues()) {
					raf.seek(value.getPosition());
					raf.write(value.getValue());
				}
			}
			raf.close();
		}
		catch(IOException e) { e.printStackTrace(); }
		orgFile = target; 
	}

	private void copy(File in, File out) throws IOException {
		if(out.exists()) out.delete();
		FileInputStream fis  = new FileInputStream(in);
		FileOutputStream fos = new FileOutputStream(out);
		byte[] buf = new byte[1024];
		int i = 0;
		while((i=fis.read(buf))!=-1) fos.write(buf, 0, i);
		fis.close();
		fos.close();
	}
}
