package opt;

import gui.MainMenu;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Scanner;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import main.TPLE_Editor;

public class Options {

	private String 	homedir		= System.getProperty("user.home");
	private String 	seperator	= System.getProperty("file.separator");

	private IniEditor	editor		= null;
	private File		optionFile	= null;
	private File		loadPath	= null;
	private File		savePath	= null;
	private boolean	hexVisible	= false;
	private boolean	checkUpdate	= true;
	private boolean	newexists	= false;
	private boolean	notified	= false;

	public Options() {
		try {
			editor = new IniEditor();

			if(!homedir.endsWith(seperator)) homedir += seperator;
			optionFile = new File(homedir + "g3_tpleditor.ini");
			loadData();
		}
		catch(IOException e) { e.printStackTrace(); }
	}

	private void loadData() throws IOException {
		if(!optionFile.exists()) optionFile.createNewFile();
		editor.load(optionFile);
		if(!editor.hasSection("Interface")) {
			createInterfaceSection();
			setHexVisible(false);
			setLoadLocation(homedir);
			setSaveLocation(homedir);
		}
		else {
			// HexEditor Visible
			if(!editor.hasOption("Interface", "HexEditorVisible")) setHexVisible(false);
			else hexVisible = Boolean.parseBoolean(editor.get("Interface", "HexEditorVisible"));

			// Load-Location
			if(!editor.hasOption("Interface", "DefaultLoadLocation")) loadPath = new File(homedir);
			else loadPath = new File(editor.get("Interface", "DefaultLoadLocation"));

			// Save-Location
			if(!editor.hasOption("Interface", "DefaultSaveLocation")) savePath = new File(homedir);
			else savePath = new File(editor.get("Interface", "DefaultSaveLocation"));
		}

		if(!editor.hasSection("Update")) {
			createUpdateSection();
			setCheckUpdate(true);
			setUpdateAviable(false);
			setUpdateNotified(false);
		}
		else {
			// check for Updates
			if(!editor.hasOption("Update", "CheckForUpdates")) setCheckUpdate(false);
			else checkUpdate = Boolean.parseBoolean(editor.get("Update", "CheckForUpdates"));

			// Update available
			if(!editor.hasOption("Update", "UpdateAviable")) setUpdateAviable(false);
			else newexists = Boolean.parseBoolean(editor.get("Update", "UpdateAviable"));

			// Notified about new Updates
			if(!editor.hasOption("Update", "Notified")) setUpdateNotified(false);
			else notified = Boolean.parseBoolean(editor.get("Update", "Notified"));
		}
	}

	public boolean isHexVisible() { return hexVisible; }
	public boolean canDoUpdate() { return checkUpdate; }
	public boolean doesNewExists() { return newexists; }
	public boolean wasNotified() { return notified; }
	public File getLoadPath() { return loadPath; }
	public File getSavePath() { return savePath; }

	public void setHexVisible(Boolean visible) {
		editor.set("Interface", "HexEditorVisible", visible.toString());
		hexVisible = visible;
		save();
	}

	public void setCheckUpdate(Boolean check) {
		editor.set("Update", "CheckForUpdates", check.toString());
		checkUpdate = check;
		save();
	}

	public void setUpdateAviable(Boolean available) {
		editor.set("Update", "UpdateAviable", available.toString());
		newexists = available;
		save();
	}

	public void setUpdateNotified(Boolean notified) {
		editor.set("Update", "Notified", notified.toString());
		this.notified = notified;
		save();
	}

	public void setLoadLocation(String path) {
		editor.set("Interface", "DefaultLoadLocation", path);
		loadPath = new File(path);
		save();
	}

	public void setSaveLocation(String path) {
		editor.set("Interface", "DefaultSaveLocation", path);
		savePath = new File(path);
		save();
	}

	private void createInterfaceSection() {
		editor.addSection("Interface");
		save();
	}

	private void createUpdateSection() {
		editor.addBlankLine("Interface");
		editor.addSection("Update");
		save();
	}

	public void save() {
		try { editor.save(optionFile); }
		catch(IOException e) { e.printStackTrace(); }
	}
	
	public boolean checkVersion() {
		try {
			URLConnection con = new URL("http://becker-print-soft.com/g3/version.php").openConnection();
			Scanner scanner = new Scanner(con.getInputStream());
			if(!TPLE_Editor.VERSION.equals(scanner.nextLine())) {
				setUpdateAviable(true);
				setUpdateNotified(true);
				con = new URL("http://becker-print-soft.com/g3/changes.php").openConnection();
				scanner = new Scanner(con.getInputStream());
				String changes = "";
				while(scanner.hasNextLine()) changes += scanner.nextLine() + "\n";
				int answer = JOptionPane.showConfirmDialog( null,
						"A new Version is available for Download!\n" + changes +
						"\nDo you want to update this Version now?",
						"Update available!",
						JOptionPane.YES_NO_OPTION,
						JOptionPane.INFORMATION_MESSAGE,
						new ImageIcon(MainMenu.class.getResource("images/updnfo.png")) );
				
				if(answer==JOptionPane.YES_OPTION) {
					String path = System.getProperty("user.dir");
					if(!path.endsWith(System.getProperty("file.separator"))) path += System.getProperty("file.separator");
					path += "g3_tpledit.jar";
					File f = new File(path);
					if(f.exists()) f.delete();
					
					con = new URL("http://becker-print-soft.com/g3/g3_tpledit.jar").openConnection();
					InputStream in = con.getInputStream();
					FileOutputStream out = new FileOutputStream(f);

					int numRead;
					byte[] buffer = new byte[1024];
					while ((numRead = in.read(buffer)) != -1) out.write(buffer, 0, numRead);
					out.flush();
					out.close();
					in.close();
					
					answer = JOptionPane.showConfirmDialog( null,
							"Update is successfully completed!\n" +
							"You must restart the Programm to work with the new Version," +
							"do you want to restart now?",
							"Update success!",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.INFORMATION_MESSAGE,
							new ImageIcon(MainMenu.class.getResource("images/updnfo.png")) );
					if(answer==JOptionPane.YES_OPTION) {
						setUpdateAviable(false);
						setUpdateNotified(false);
						Runtime.getRuntime().exec("cmd.exe /C " + "\"" + f.getAbsolutePath() + "\"");
						System.exit(0);
					}
				}
				return true;
			}
		}
		catch(IOException e) { e.printStackTrace(); }
		return false;
	}
}
